//
// Copyright (c) 2002
// Ronald Kevin Burton
//
// This material is provided "as is", with absolutely no warranty expressed
// or implied. This software is provided solely as part of ".NET Common Language
// Runtime Unleashed" to help illustrate concepts and principles described in 
// that book. Any use is at your own risk.
//
// Permission to use or copy this software for any purpose is hereby granted 
// without fee, provided the above notices are retained on all copies.
// Permission to modify the code and to distribute modified code is granted,
// provided the above notices are retained, and a notice that the code was
// modified is included with the above copyright notice.
//
// 
using System;
using System.Collections;
using System.Threading;

namespace DelegatesAndEvents
{
	public enum ChangeType {Add, Subtract}

	public class ChangedClass 
	{
		// An event that clients can use to be notified whenever the
		// elements of the list change:
		public delegate void ChangeHandler(ChangeType type, int count);
		public event ChangeHandler Changed;
		private int count;
		public ChangedClass()
		{
			count = 0;
		}
		// Invoke the Changed event; called whenever list changes:
		protected void OnChanged(ChangeType type, int count) 
		{
			if (Changed != null)
			{
				Changed(type, count);
			}
		}
		public void Add()
		{
			Interlocked.Increment(ref count);
			OnChanged(ChangeType.Add, count);
		}
		public void Subtract()
		{
			Interlocked.Decrement(ref count);
			OnChanged(ChangeType.Subtract, count);
		}
	}

	class Test 
	{
		private static void Changed(ChangeType type, int count) 
		{
			Console.WriteLine("Changed {0} {1} fired", count, type);
		}
		private static void Modified(ChangeType type, int count) 
		{
			Console.WriteLine("Modified {0} {1} fired", count, type);
		}
		public static void Main() 
		{
			// Create a new list:
			ChangedClass changed = new ChangedClass();
			changed.Changed += new ChangedClass.ChangeHandler(Changed);
			changed.Changed += new ChangedClass.ChangeHandler(Modified);
			changed.Add();
			changed.Add();
			changed.Changed -= new ChangedClass.ChangeHandler(Modified);
			changed.Add();
			changed.Add();
			changed.Subtract();
			changed.Subtract();
			changed.Subtract();
			// Delegate [] da = changed.Changed.GetInvocationList();
		}
	}
}
