//
// Copyright (c) 2002
// Ronald Kevin Burton
//
// This material is provided "as is", with absolutely no warranty expressed
// or implied. This software is provided solely as part of ".NET Common Language
// Runtime Unleashed" to help illustrate concepts and principles described in 
// that book. Any use is at your own risk.
//
// Permission to use or copy this software for any purpose is hereby granted 
// without fee, provided the above notices are retained on all copies.
// Permission to modify the code and to distribute modified code is granted,
// provided the above notices are retained, and a notice that the code was
// modified is included with the above copyright notice.
//
// 
using System;

namespace DelegatesAndEvents
{
	delegate void DelegateCallback();
	class A
	{
		public void ACallback()
		{
			Console.WriteLine("A.ACallback()");
		}
		public void BCallback()
		{
			Console.WriteLine("A.BCallback()");
		}
		public void CCallback()
		{
			Console.WriteLine("A.CCallback()");
		}
	}
	class B
	{
		public void ACallback()
		{
			Console.WriteLine("B.ACallback()");
		}
		public void BCallback()
		{
			Console.WriteLine("B.BCallback()");
		}
		public void CCallback()
		{
			Console.WriteLine("B.CCallback()");
		}
	}
	class C
	{
		public void ACallback()
		{
			Console.WriteLine("C.ACallback()");
		}
		public void BCallback()
		{
			Console.WriteLine("C.BCallback()");
		}
		public void CCallback()
		{
			Console.WriteLine("C.CCallback()");
		}
	}
	class DelegateEquality
	{
		static void ACallback()
		{
			Console.WriteLine("ACallback()");
		}
		static void BCallback()
		{
			Console.WriteLine("BCallback()");
		}
		static void CCallback()
		{
			Console.WriteLine("CCallback()");
		}
		static void Main(string[] args)
		{
			DelegateCallback ad = new DelegateCallback(ACallback);
			DelegateCallback bd = new DelegateCallback(BCallback);
			DelegateCallback cd = new DelegateCallback(ACallback);
			Console.WriteLine("static {0} == {1} {2}", ad.Method, bd.Method, ad == bd);
			Console.WriteLine("static {0} == {1} {2}", ad.Method, ad.Method, ad == ad);
			Console.WriteLine("static {0} == {1} {2}", ad.Method, cd.Method, ad == cd);
			
			A a1 = new A();
			B b1 = new B();
			C c1 = new C();

			ad = new DelegateCallback(a1.ACallback);
			bd = new DelegateCallback(a1.BCallback);
			Console.WriteLine("instance {0}.{1} == {2}.{3} {4}", ad.Target, ad.Method, bd.Target, bd.Method, ad == bd);
			cd = new DelegateCallback(a1.ACallback);
			Console.WriteLine("instance {0}.{1} == {2}.{3} {4}", ad.Target, ad.Method, cd.Target, cd.Method, ad == cd);
			
			A a2 = new A();

			bd = new DelegateCallback(a2.ACallback);
			Console.WriteLine("instance {0}.{1} == {2}.{3} {4}",  ad.Target, ad.Method, bd.Target, bd.Method, ad == bd);

			DelegateCallback dd = ad;
			dd += bd;
			dd += cd;

			DelegateCallback ed = ad;
			ed += bd;
			ed += cd;
			Console.WriteLine("chain {0} == {1} {2}", dd.Method, ed.Method, dd == ed);

			ed = cd;
			ed += bd;
			ed += ad;
			Console.WriteLine("chain {0} == {1} {2}", dd.Method, ed.Method, dd == ed);
		}
	}
}
