//
// Copyright (c) 2002
// Ronald Kevin Burton
//
// This material is provided "as is", with absolutely no warranty expressed
// or implied. This software is provided solely as part of ".NET Common Language
// Runtime Unleashed" to help illustrate concepts and principles described in 
// that book. Any use is at your own risk.
//
// Permission to use or copy this software for any purpose is hereby granted 
// without fee, provided the above notices are retained on all copies.
// Permission to modify the code and to distribute modified code is granted,
// provided the above notices are retained, and a notice that the code was
// modified is included with the above copyright notice.
//
// 
using System;
using System.Collections;
using System.Threading;

namespace DelegatesAndEvents
{
	public enum ChangeType {Add, Subtract}

	public class ChangedEventArgs : EventArgs 
	{
		private ChangeType _type;
		private int _value;
		public ChangedEventArgs(ChangeType _type, int _value)
		{
			this._type = _type;
			this._value = _value;
		}
		public ChangeType Type
		{
			get
			{
				return _type;
			}
		}
		public int Value
		{
			get
			{
				return _value;
			}
		}
	}

	public class ChangedClass 
	{
		// An event that clients can use to be notified whenever the
		// elements of the list change:
		public event EventHandler Changed;
		private int count;
		public ChangedClass()
		{
			count = 0;
		}
		// Invoke the Changed event; called whenever list changes:
		protected void OnChanged(EventArgs e) 
		{
			if (Changed != null)
				Changed(this, e);
		}
		public void Add()
		{
			Interlocked.Increment(ref count);
			OnChanged(new ChangedEventArgs(ChangeType.Add, count));
		}
		public void Subtract()
		{
			Interlocked.Decrement(ref count);
			OnChanged(new ChangedEventArgs(ChangeType.Subtract, count));
		}
	}

	class Test 
	{
		private static void Changed(object sender, EventArgs e) 
		{
			Console.WriteLine("Changed {0} {1} fired",
				              ((ChangedEventArgs)e).Value,
				              ((ChangedEventArgs)e).Type);
		}
		private static void Modified(object sender, EventArgs e) 
		{
			Console.WriteLine("Modified {0} {1} fired",
				              ((ChangedEventArgs)e).Value,
				              ((ChangedEventArgs)e).Type);
		}
		// Test the EventHandler class:
		public static void Main() 
		{
			// Create a new list:
			ChangedClass changed = new ChangedClass();
			changed.Changed += new EventHandler(Changed);
			changed.Changed += new EventHandler(Modified);
			changed.Add();
			changed.Add();
			changed.Changed -= new EventHandler(Modified);
			changed.Add();
			changed.Add();
			changed.Subtract();
			changed.Subtract();
			changed.Subtract();
		}
	}
}
