//
// Copyright (c) 2002
// Ronald Kevin Burton
//
// This material is provided "as is", with absolutely no warranty expressed
// or implied. This software is provided solely as part of ".NET Common Language
// Runtime Unleashed" to help illustrate concepts and principles described in 
// that book. Any use is at your own risk.
//
// Permission to use or copy this software for any purpose is hereby granted 
// without fee, provided the above notices are retained on all copies.
// Permission to modify the code and to distribute modified code is granted,
// provided the above notices are retained, and a notice that the code was
// modified is included with the above copyright notice.
//
// 
using System;
using System.Reflection;

namespace DelegatesAndEvents
{
	class DelegateClass
	{
		public void HelloWorldProcessor(string message)
		{
			Console.WriteLine("DelegateClass: {0}", message);
		}
		public void A(string message)
		{
			Console.WriteLine("A: {0}", message);
		}
		public void B(string message)
		{
			Console.WriteLine("B: {0}", message);
		}
		public void C(string message)
		{
			Console.WriteLine("C: {0}", message);
		}
	}
	class HelloWorld
	{
		// static private int HelloWorldProcessor(string message)
		static private void HelloWorldProcessor(string message)
		{
			Console.WriteLine(message);
		}
		public delegate void HelloWorldDelegate(string message);
		static void Main(string[] args)
		{
			HelloWorldDelegate d = new HelloWorldDelegate(HelloWorldProcessor);
			d("Hello World!");

			Console.WriteLine("Method: {0}", d.Method);
			Console.WriteLine("Target: {0}", d.Target);

			DelegateClass dc = new DelegateClass();
			d = new HelloWorldDelegate(dc.HelloWorldProcessor);
			d("Hello World!");
			IAsyncResult ar = d.BeginInvoke("Invoked Hello World!", null, null);

			Console.WriteLine("Method: {0}", d.Method);
			Console.WriteLine("Target: {0}", d.Target);

			d.EndInvoke(ar);

			d = new HelloWorldDelegate(HelloWorldProcessor);
			d += new HelloWorldDelegate(dc.HelloWorldProcessor);
			d += new HelloWorldDelegate(dc.A);
			d += new HelloWorldDelegate(dc.B);
			d += new HelloWorldDelegate(dc.C);
			d("Multicast Hello World!");

			Console.WriteLine("Method: {0}", d.Method);
			Console.WriteLine("Target: {0}", d.Target);

			Delegate [] list = d.GetInvocationList();
			foreach(Delegate item in list)
			{
				Console.WriteLine("Method: {0}", item.Method);
				Console.WriteLine("Target: {0}", item.Target);
			}

			Type t = dc.GetType();
			MethodInfo mi = t.GetMethod("HelloWorldProcessor");
			ParameterInfo[] pia = mi.GetParameters();
			foreach(ParameterInfo pi in pia)
			{
				Console.WriteLine(pi.ParameterType);
			}

			d = (HelloWorldDelegate)Delegate.CreateDelegate(typeof(HelloWorldDelegate), dc, "HelloWorldProcessor"); 
			d.DynamicInvoke(new object [] {"Late-Bound Hello World!"});
			d = (HelloWorldDelegate)Delegate.CreateDelegate(typeof(HelloWorldDelegate), typeof(HelloWorld), "HelloWorldProcessor"); 
			d.DynamicInvoke(new object [] {"Late-Bound static Hello World!"});

		}
	}
}
